/**
 * Copyright (c) 2015, STARSCHEMA LTD.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This grammar implement the sql select statement
 *
 *    @author Attila Horvath
 *    @author Balazs Gunics
 *
 * Modified by
 * @author  Data Virtuality GmbH
 * @date    $Date: 2015-04-14 00:39:27 +0200 (Вт, 14 апр 2015) $
 */

grammar JdbcGrammar;

options {
  language = Java;
  output = AST;
}
import JDBCTokens;

@header {/**
 * Starschema Big Query JDBC Driver
 * Copyright (C) 2012, Starschema Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * This grammar implement the sql select statement
 *    @author Horvath Attila
 *    @author Balazs Gunics
 */
  package net.starschema.clouddb.jdbc;
}
@lexer::header{/**
 * Starschema Big Query JDBC Driver
 * Copyright (C) 2012, Starschema Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * This grammar implement the sql select statement
 *    @author Horvath Attila
 *    @author Balazs Gunics
 */
  package net.starschema.clouddb.jdbc;
}

/**
Rule for entry point for the parser makes the selectstatement as the root for the Tree
*/
statement
:
  e=selectstatement^ (EOF | ';')!
;

/**
Rule for parsing an sql select
*/
selectstatement
:   SELECTKEYWORD DISTINCT? expression fromexpression whereexpression? groupbyexpression? havingexpression? orderbyexpression? limitexpression?
    ->^(SELECTSTATEMENT SELECTKEYWORD expression fromexpression whereexpression? groupbyexpression? havingexpression? orderbyexpression? limitexpression?)

;

fragment LIMITEXPRESSION:;
/**
BIGQUERY LIMIT expression
*/
limitexpression
:
    LIMITKEYWORD NUMBER ->^(LIMITEXPRESSION NUMBER)
;

/**
Rule for parsing a whole order by expression, creates Tree node ORDERBYEXPRESSION
*/
orderbyexpression
:
    (
    ORDERKEYWORD BYKEYWORD orderbycondition(COMMA orderbycondition)*
    )->^(ORDERBYEXPRESSION orderbycondition*)
;

fragment ORDERBYCLAUSE:;
fragment ORDERBYORDER:;
/**
Rule for parsing a single condition for an order by expression
*/
orderbycondition
:
    column (DESC|ASC)? ->^(ORDERBYCLAUSE column* ^(ORDERBYORDER DESC? ASC?))
;

/**
Rule for parsing the expression followed by the FROMKEYWORD, creates Tree node FROMEXPRESSION
*/
fromexpression
:
    FROMKEYWORD
    datasource
    (
    COMMA
        datasource
    )*
    ->^(FROMEXPRESSION datasource* ^(TEXT TEXT[$fromexpression.text]))
;

/**
Rule for parsing logical operators
*/
logicaloperator
:
    (andoperator|oroperator)->^(LOGICALOPERATOR andoperator? oroperator? ^(TEXT TEXT[$logicaloperator.text]))
;

andoperator
:
  AND
;

oroperator
:
  OR
;


/**
Rule to parse a where condition expression, creates Tree node WHEREEXPRESSION and nested BOOLEANEXPRESSIONS
*/
whereexpression
:
    (WHEREKEYWORD expr
    )->^(WHEREEXPRESSION
                    expr
            ^(TEXT TEXT[$whereexpression.text])
        )
;

expr
:
  (disjunction)=>disjunction
  |(conjunction)=>conjunction
  |(subconjunction)=>subconjunction
  |(subdisjunction)=>subdisjunction
  |(exprlvl0)=>exprlvl0
;

subdisjunction
:
  (NOTKEYWORD LPARAM subdisjunction RPARAM)=>(NOTKEYWORD LPARAM subdisjunction RPARAM)->^(NEGATION subdisjunction)
  |(NOTKEYWORD LPARAM disjunction RPARAM)=>(NOTKEYWORD LPARAM disjunction RPARAM)->^(NEGATION disjunction)
  |(LPARAM! subdisjunction RPARAM!)=>(LPARAM! subdisjunction RPARAM!)
  |(LPARAM! disjunction RPARAM!)=>(LPARAM! disjunction RPARAM!)
;

disjunctionpart1
:
  (conjunction)=>conjunction
  |(subconjunction)=>subconjunction
  |(subdisjunction)=>subdisjunction
  |exprlvl0
;

fragment DISJUNCTION:;
disjunction
:
  (disjunctionpart1 disjunctionpart2+)->^(DISJUNCTION disjunctionpart1 disjunctionpart2*)
;

disjunctionpart2
:
  oroperator disjunctionpart1
;

conjunctionpart1
:
  (subconjunction)=>subconjunction
  |(subdisjunction)=>subdisjunction
  |exprlvl0
;

subconjunction
:
  (NOTKEYWORD LPARAM subconjunction RPARAM)=>(NOTKEYWORD LPARAM subconjunction RPARAM)->^(NEGATION subconjunction)
  |(NOTKEYWORD LPARAM conjunction RPARAM)=>(NOTKEYWORD LPARAM conjunction RPARAM)->^(NEGATION conjunction)
  |(LPARAM! subconjunction RPARAM!)=>(LPARAM! subconjunction RPARAM!)
  |(LPARAM! conjunction RPARAM!)=>(LPARAM! conjunction RPARAM!)
;
fragment CONJUNCTION:;
conjunction
:
  (conjunctionpart1 conjunctionpart2+)->^(CONJUNCTION conjunctionpart1 conjunctionpart2*)
;

conjunctionpart2
:
  andoperator conjunctionpart1
;

fragment NEGATION:;
exprlvl0
:
  (NOTKEYWORD LPARAM exprlvl0 RPARAM)=>(NOTKEYWORD LPARAM exprlvl0 RPARAM)->^(NEGATION exprlvl0)
  |(NOTKEYWORD LPARAM exprlvl1 RPARAM)=>(NOTKEYWORD LPARAM exprlvl1 RPARAM)->^(NEGATION exprlvl1)
  |(LPARAM! exprlvl0 RPARAM!)=>(LPARAM! exprlvl0 RPARAM!)
  |(LPARAM! exprlvl1 RPARAM!)=>(LPARAM! exprlvl1 RPARAM!)
  |(exprlvl1)
;

exprlvl1
:
   exprcondition
;

fragment BOOLEANEXPRESSIONITEMLEFT:;
fragment BOOLEANEXPRESSIONITEMRIGHT:;
fragment LIKEEXPRESSION:;
fragment BOOLEANEXPRESSIONITEM:;
/**
Rule to parse a condition expression used in HAVING and WHERE, creates Tree node BOOLEANEXPRESSION and nested column? subquery? comparisonoperator? column? subquery?
STILL NEED TO IMPLEMENT LOT
*/
exprcondition
:
    (
      s1=exprconditioncore
      (
          (LIKEKEYWORD likeclause)
      |   (comparisonoperator
              (s2=exprconditioncore|s3=number)

          )
      |   (s4=isnullornot)
      )
  )->^(BOOLEANEXPRESSIONITEM ^(BOOLEANEXPRESSIONITEMLEFT $s1) comparisonoperator?  ^(BOOLEANEXPRESSIONITEMRIGHT likeclause? $s2? $s3? $s4?) ^(TEXT TEXT[$exprcondition.text]))
;

isnullornot
:
(ISKEYWORD NOTKEYWORD? NULLKEYWORD)
->^(ISNULLEXPRESSION NOTKEYWORD?)
;

exprconditioncore:
subquery|columnforexpression|mystringliteral|functioncall
;

columnforexpression
:
  (
   (razoralias COLON)?
   (scopealias (COLON | PUNCTUATION))*
   (srcname) 
   ( alias)?
  ) ->^(COLUMN scopealias* srcname alias? ^(TEXT TEXT[$columnforexpression.text]))
;

/**
Rule to parse a having condition expression, creates Tree node HAVINGEXPRESSION and nested BOOLEANEXPRESSIONS
*/
havingexpression
:
    (
    HAVINGKEYWORD expr
    )->^(HAVINGEXPRESSION
                    expr
                    ^(TEXT TEXT[$havingexpression.text])
        )
;



/**
Rule to parse a Group By clause, creates Tree node GROUPBYEXPRESSION
*/
groupbyexpression
:
   (GROUPKEYWORD BYKEYWORD columnforsingleexpression (COMMA columnforsingleexpression)*) ->^(GROUPBYEXPRESSION columnforsingleexpression+ ^(TEXT TEXT[$groupbyexpression.text]))
;

fragment CONDITIONLEFT:;
fragment CONDITIONRIGHT:;


/**
Rule for parsing condition of a JOIN, creates Tree node CONDITION
*/
condition
:
 (
    s1=column comparisonoperator s2=column
 )->^(CONDITION ^(CONDITIONLEFT $s1) comparisonoperator ^(CONDITIONRIGHT $s2) ^(TEXT TEXT[$condition.text]))
;

/**
Rule for parsing a subselect
*/

subquery
:
   (
    LPARAM
      selectstatement
    RPARAM
    (alias)?
   )
   ->^(SUBQUERY selectstatement alias? ^(TEXT TEXT[$subquery.text]))
;

fragment LEFTEXPR:;
fragment RIGHTEXPR:;
fragment MULTIJOINEXPRESSION:;
fragment PARENJOIN:;

datasourceelement
:
  (subquery | sourcetable)
;

datasource
:
  datasourcenoparen
  |
  //(LPARAM+ datasourceelement joinclause) =>
  datasourceparen
;

/**
*Rule for parsing datasources after FROMKEYWORD
*/
datasourcenoparen
@init{boolean joinexpr = false;}
:
   (s1=datasourceelement (joinclause1=joinclause joinelement1=datasourceelement onclause1=onclause (multijoinexpression)* {joinexpr=true;})?)
   -> {joinexpr}?  ^(JOINEXPRESSION
                        ^(LEFTEXPR $s1?)
                        $joinclause1?
                        ^(RIGHTEXPR $joinelement1?) $onclause1? multijoinexpression*
                    )
   -> $s1
;

/**Because of SAP crystal report */
datasourceparen
:
(
  //1-2
                (LPARAM simplejoin RPARAM) multijoinexpression?
 |//2-3
        (LPARAM (LPARAM simplejoin RPARAM) multijoinexpression RPARAM)  multijoinexpression?
 |//3-4
(LPARAM (LPARAM (LPARAM simplejoin RPARAM) multijoinexpression RPARAM)  multijoinexpression RPARAM)
multijoinexpression?
 |//4-5
(LPARAM
(LPARAM (LPARAM (LPARAM simplejoin RPARAM) multijoinexpression RPARAM)  multijoinexpression RPARAM)
multijoinexpression RPARAM) multijoinexpression?
 |//5-6
(LPARAM (LPARAM
(LPARAM (LPARAM (LPARAM simplejoin RPARAM) multijoinexpression RPARAM)  multijoinexpression RPARAM)
multijoinexpression RPARAM) multijoinexpression RPARAM) multijoinexpression?
 |//6-7
(LPARAM (LPARAM (LPARAM
(LPARAM (LPARAM (LPARAM simplejoin RPARAM) multijoinexpression RPARAM)  multijoinexpression RPARAM)
multijoinexpression RPARAM) multijoinexpression RPARAM) multijoinexpression RPARAM) multijoinexpression?
 |//7-8
(LPARAM (LPARAM (LPARAM (LPARAM
(LPARAM (LPARAM (LPARAM simplejoin RPARAM) multijoinexpression RPARAM)  multijoinexpression RPARAM)
multijoinexpression RPARAM) multijoinexpression RPARAM) multijoinexpression RPARAM) multijoinexpression RPARAM)
multijoinexpression?
 |//8-9
(LPARAM (LPARAM (LPARAM (LPARAM (LPARAM
(LPARAM (LPARAM (LPARAM simplejoin RPARAM) multijoinexpression RPARAM)  multijoinexpression RPARAM)
multijoinexpression RPARAM) multijoinexpression RPARAM) multijoinexpression RPARAM) multijoinexpression RPARAM)
multijoinexpression RPARAM) multijoinexpression?
 |//9-10
(LPARAM (LPARAM (LPARAM (LPARAM (LPARAM (LPARAM
(LPARAM (LPARAM (LPARAM simplejoin RPARAM) multijoinexpression RPARAM)  multijoinexpression RPARAM)
multijoinexpression RPARAM) multijoinexpression RPARAM) multijoinexpression RPARAM) multijoinexpression RPARAM)
multijoinexpression RPARAM) multijoinexpression RPARAM) multijoinexpression?
 |//10-11
(LPARAM (LPARAM (LPARAM (LPARAM (LPARAM (LPARAM (LPARAM
(LPARAM (LPARAM (LPARAM simplejoin RPARAM) multijoinexpression RPARAM)  multijoinexpression RPARAM)
multijoinexpression RPARAM) multijoinexpression RPARAM) multijoinexpression RPARAM) multijoinexpression RPARAM)
multijoinexpression RPARAM) multijoinexpression RPARAM) multijoinexpression RPARAM) multijoinexpression?


 )  ->^(JOINEXPRESSION simplejoin multijoinexpression* )
;

simplejoin
:
  (s1=datasourceelement joinclause1=joinclause joinelement1=datasourceelement onclause1=onclause )
  -> ^(LEFTEXPR $s1?) $joinclause1? ^(RIGHTEXPR $joinelement1?) $onclause1?
;

datasourcerecur
:
  datasourceelement  //table | subquery
  |
  LPARAM datasourcerecur multijoinexpression RPARAM
;


multijoinexpression
:
  joinclause datasourceelement onclause
  ->
  ^(MULTIJOINEXPRESSION
                        joinclause
                        ^(RIGHTEXPR datasourceelement) onclause
                    )
;

/**
Rule for parsing the Type of a join creates Tree node JOINTYPE (Keyword)
*/
joinclause
:
    (jointypes? JOINKEYWORD EACH?)->^(JOINTYPE jointypes? JOINKEYWORD EACH? ^(TEXT TEXT[$joinclause.text]))
;


/**
Rule for parsing the Condition of a join
*/
onclause
:
(ONKEYWORD
            (
                (LPARAM condition RPARAM) | condition
            )
            )->^(ONCLAUSE condition ^(TEXT TEXT[$onclause.text]))
;



/**
Rule for parsing dataset name in datasource of referred column or table, creates Tree node DATASETNAME
*/
dataset
:
  (  s1=quotedname 
   | (ESCAPEDDOUBLEQUOTE s2=name ESCAPEDDOUBLEQUOTE) 
   | (BACKQUOTE s2=name BACKQUOTE)
   | s2=name
  )
  ->^(DATASETNAME $s1? $s2?)
;

/**
Rule for parsing Project name in datasource of referred column or table, creates Tree node PROJECTNAME
*/
project
:
  ( options {greedy=true;} :
    (SINGLEQUOTE (name projectdivider? )+ SINGLEQUOTE)
   | (DOUBLEQUOTE (name projectdivider? )+ DOUBLEQUOTE)
   | (ESCAPEDDOUBLEQUOTE (name projectdivider? )+ ESCAPEDDOUBLEQUOTE)
   | name
  )
    ->^(PROJECTNAME name (projectdivider name)* )
;
/**
Rule for string dividers. When strings surronded by quotes, they may contain special characters like:
<li> .
<li> :
*/
projectdivider
:
  ( COLON | PUNCTUATION ) ->^(DIVIDER DIVIDER[$projectdivider.text])
;

/**
Rule for parsing Source Tables Creates Tree node SOURCETABLE with nested project,dataset,name,alias
*/
sourcetablebase
:
  (
    (
     (((project columndivider)? dataset columndivider)? srcname) 
       ( alias)?
    )->^(SOURCETABLE project? dataset? srcname alias? columndivider*  ^(TEXT TEXT[$sourcetablebase.text]))
  ) | (
  	( BRACE_QUOTED_STRING ( alias)?
  	)->^(SOURCETABLE1 BRACE_QUOTED_STRING alias? ^(TEXT TEXT[$sourcetablebase.text]))
  )
;

wrappedsourcetablebase
:
(LPARAM
TABLE_DATE_RANGE
LPARAM BRACE_QUOTED_STRING
COMMA s1=singleexpression
COMMA s2=singleexpression
RPARAM
RPARAM
) -> ^(SOURCETABLE1 BRACE_QUOTED_STRING ^(TABLE_DATE_RANGE $s1 $s2))
;

sourcetable
:
sourcetablebase
|
wrappedsourcetablebase
;

srcname:
quotedname
| (ESCAPEDDOUBLEQUOTE name ESCAPEDDOUBLEQUOTE)
| (BACKQUOTE name BACKQUOTE)
| name
;


quotedname
:
DOUBLE_QUOTED_STRING->^(NAMEQ DOUBLE_QUOTED_STRING)
;

/**
Rule for naming structures such as columns,tables etc creates Tree node NAME
*/
name
:
 IDENTIFIER ->^(NAME IDENTIFIER)
;


/** Contains 1 or more number(0..9) */
number
:
   NUMBER->^(INTEGERPARAM NUMBER)
   |
   FLOAT->^(FLOATPARAM FLOAT)
;


fragment JOKERCALL:;
/**
Rule for parsing the expression before from after select, creates Tree node EXPRESSION and nested column,functioncall
*/
expression
:
   '*' ->^(EXPRESSION ^(JOKERCALL '*') ^(TEXT TEXT[$expression.text]))
   |
   (
        ( singleexpression )
        ( COMMA singleexpression )*
   )->^(EXPRESSION singleexpression* ^(TEXT TEXT[$expression.text]))
;

singleexpression
:
(
	expressionresume
	( withinrecord)?
	( alias)?
)->^(SINGLEEXPRESSION expressionresume withinrecord? alias?)
;


/**
Rule for identifying a * call from any tables it creates a Tree node named MULTIPLECALL
*/
multiplecolumn
:
    (
     (scopealias PUNCTUATION)+
     '*'
    )->^(MULTIPLECALL (scopealias)* PUNCTUATION* '*' ^(TEXT TEXT[$multiplecolumn.text]))
;
fragment SCOPE:;
scopealias
:
(IDENTIFIER |
(BACKSINGLEQUOTE IDENTIFIER BACKSINGLEQUOTE) |
(DOUBLEQUOTE IDENTIFIER DOUBLEQUOTE)
 )->^(SCOPE IDENTIFIER)
;

//for razorSQLs projectname which we won't need
razoralias
:
DOUBLEQUOTE IDENTIFIER ((COLON | PUNCTUATION) IDENTIFIER)+ DOUBLEQUOTE
;

BACKSINGLEQUOTE:
'`'
;

fragment WITHINRECORD:;

/**
    Rule for identifying an sql function creates tree elements FUNCTIONCALL and nested FUNCTIONPARAMETERS,name,alias and TEXT
*/
functioncall
:
  (
   name LPARAM DISTINCT? functionparameters RPARAM
   ( alias)?
  )->^(FUNCTIONCALL  name functionparameters alias? ^(TEXT TEXT[$functioncall.text]))
;

functioncallinternal
:
  (
	name LPARAM DISTINCT? functionparameters RPARAM
  )->^(FUNCTIONCALL  name functionparameters ^(TEXT TEXT[$functioncallinternal.text]))
;

withinrecord
:
(WITHINKEYWORD RECORDKEYWORD)->^(WITHINRECORD WITHINRECORD)
;

/**   Paramaters for sql functions    */
functionparameters
:
         //nothing (empty parameter list)
        |(
            (singleexpressionfunc)
            ((COMMA|ASKEYWORD) singleexpressionfunc)*
        )
        | joker
;

fragment JOKER:;
joker:
('*')->^(SINGLEEXPRESSION ^(JOKER JOKER))
;

fragment DTTYPE:;
dttype:
(TYPEDATE|TYPETIME|TYPEDATETIME)->^(DTTYPE ^(TEXT[$dttype.text]))
;

singleexpressionfunc
:
expressionresume->^(SINGLEEXPRESSION expressionresume)
;

expressionresume
:
functionexpressionparameter (expressionoperator functionexpressionparameter)*
;

functionexpressionparameter
:
(simplefunctionparameter ((comparisonoperator simplefunctionparameter) | isnullornotforsingleexpression)?)
;

simplefunctionparameter
:
(mystringliteral | columnforsingleexpression | number | booleanparam | nullnode | functioncallinternal | (LPARAM expressionresume RPARAM) | dttype)
;

isnullornotforsingleexpression
:
(ISKEYWORD NOTKEYWORD? NULLKEYWORD)
->^(NULLNODE TEXT[$isnullornotforsingleexpression.text])
;

columnforsingleexpression
:
 (
    ((scopealias | quotedname) (COLON | PUNCTUATION) )*
   (s1=name | s2=quotedname) 
  ) ->^(COLUMN scopealias* $s1? $s2? ^(TEXT TEXT[$columnforsingleexpression.text]))
;

nullnode
:
NULLKEYWORD->^(NULLNODE TEXT[$nullnode.text])
;

expressionoperator:
  expressionoperatorbase ->^(EXPRESSIONOPERATOR expressionoperatorbase)
; 

booleanparam
:
(s1=(TRUEKEYWORD | FALSEKEYWORD))->^(BOOLEANPARAM $s1)
;

expressionoperatorbase:
(
  AND  |
  OR  |
  '+'  |
  '-' | 
  '*' |
  '/'  |
  '%'  |
  '&' |
  '|' |
  '^' |
  '<<' |
  '>>' |
  '~' |
  '||'
  )
;

/** SingelQuote stringliteral for functionparamteres */
mystringliteral
:
SINGLE_QUOTED_STRING->^(STRINGLIT SINGLE_QUOTED_STRING)
;

/** types of join */
jointypes: ( INNERKEYWORD  | LEFT_KEYWORD | RIGHT_KEYWORD | FULl_KEYWORD ) ;

/**
Rule for parsing comparison operators
*/
comparisonoperator:
  comparisonoperatorbase ->^(COMPARISONOPERATOR comparisonoperatorbase)
;

comparisonoperatorbase:
(
  '='  |
  '<>' |
  '!=' |
  '>'  |
  '<'  |
  '>=' |
  '<='
  )
;




//--------------------------------------------------------------------------------------------
//Hidden things
/**    WhiteSpace Characters (Hidden)*/
WS
:
    (' ' | '\t' | '\n' | '\r' | '\f' | '%n' )+ { $channel = HIDDEN;}
;

/** We don't care about comments */
COMMENT
    :   (   Start_Comment ( options {greedy=false;} : . )* End_Comment )+
    {
      $channel=HIDDEN;
    }
    ;
/** We don't care abut line comments */
LINE_COMMENT
    :   (   ( Line_Comment | '--' ) ~('\n'|'\r')* '\r'? '\n')+
    {
      $channel=HIDDEN;
    }
    ;
fragment KEYWORD:;
    /**
Used for renames for example when you refer a column by not on its original name, creates Tree node ALIAS
*/
alias
:

	    (

	        ASKEYWORD IDENTIFIER ->^(ALIAS IDENTIFIER ^(KEYWORD ASKEYWORD) ^(TEXT TEXT[$alias.text]))
	    |   ASKEYWORD aliasliteral ->^(ALIAS aliasliteral ^(KEYWORD ASKEYWORD) ^(TEXT TEXT[$alias.text]))
	    )
    |
	    (
	        IDENTIFIER ->^(ALIAS IDENTIFIER ^(TEXT TEXT[$alias.text]))
	    |   aliasliteral ->^(ALIAS aliasliteral ^(TEXT TEXT[$alias.text]))
	    )
;

aliasliteral
:
(SINGLEQUOTE s1=~(SINGLEQUOTE)* SINGLEQUOTE)
| (DOUBLEQUOTE IDENTIFIER DOUBLEQUOTE)
;

/**
Rule for identifying any columns in the syntax, creates Tree node COLUMN and nested project,dataset,table,name,alias
<p>is it starts with
<li> project.dataset.table.
<li> dataset.table.
<li> table.
</p><p> followed by
<li> name
<li> optionally with alias

*/
column
:
 (
    (scopealias (COLON | PUNCTUATION) )*
   (name | (DOUBLEQUOTE name DOUBLEQUOTE))// | (ESCAPEDDOUBLEQUOTE name ESCAPEDDOUBLEQUOTE))
   (alias)?
  ) ->^(COLUMN scopealias* name alias? ^(TEXT TEXT[$column.text]))
;

/** String: '.' or ':' */
columndivider: COLON | PUNCTUATION ;


/**
Rule to parse an sql like expression
*/
likeclause
:
    mystringliteral -> ^(LIKEEXPRESSION mystringliteral)
;

/**
Rule for parsing table name in datasource of referred column or table, creates Tree node TABLENAME
*/
table
:
   (   (BACKQUOTE IDENTIFIER BACKQUOTE) |
      (SINGLEQUOTE IDENTIFIER SINGLEQUOTE)
    |  (DOUBLEQUOTE IDENTIFIER DOUBLEQUOTE)
    |  (ESCAPEDDOUBLEQUOTE IDENTIFIER ESCAPEDDOUBLEQUOTE)
    |   IDENTIFIER
   )
   ->^(TABLENAME SINGLEQUOTE? DOUBLEQUOTE? ESCAPEDDOUBLEQUOTE? BACKQUOTE? ^(NAME IDENTIFIER) SINGLEQUOTE? DOUBLEQUOTE? ESCAPEDDOUBLEQUOTE? BACKQUOTE?)
;