/**
 * Copyright (c) 2015, STARSCHEMA LTD.
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:

 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Modified by
 * @author  Data Virtuality GmbH
 * @date    $Date$
 */
package net.starschema.clouddb.jdbc.list;

import java.util.ArrayList;
import java.util.Arrays;

import java.util.List;
import net.starschema.clouddb.jdbc.BQSupportFuncts;
import net.starschema.clouddb.jdbc.JdbcGrammarParser;
import net.starschema.clouddb.jdbc.antlr.sqlparse.TreeParsingException;

import org.antlr.runtime.tree.Tree;

/**
 * This class extends the basic Node with a refobject to store the
 * FunctionParameters <li>column <li>integer <li>string
 *
 * @author Attila Horvath, Balazs Gunics
 */
// EXTENSION
// ISSUE: DVCORE-2780
// FIXED BY: Roman Pedchenko
// DATE: 2015.04.09
// public class FunctionParameter extends Node implements ColumnReferencePlace {
public class FunctionParameter extends Node implements ColumnReferencePlace, UniQueIdContainer {
// END EXTENSION
    
    Node refObject;
    SelectStatement selectStatement;

    TreeBuilder builder;
    
	// EXTENSION
	// ISSUE: DVCORE-2780
	// FIXED BY: Roman Pedchenko
	// DATE: 2015.04.09
    String alias = null;
    String uniqueId = null;
	private Node parentNode;
	private boolean withinrecord = false;

	// END EXTENSION
    /**
     * Constructor for the FunctionParameter which builds up the parameter
     * from the ANTLR tree
     *
     * @param t - the ANTLR tree
     * @param treeBuilder - TreeBuilder for the helper functions
     * @param selectStatement - the selectStatement which contains the Function
     * @throws TreeParsingException
     */
    // EXTENSION
    // ISSUE: DVCORE-2780
    // FIXED BY: Roman Pedchenko
    // DATE: 2015.04.09
    // public FunctionParameter(Tree t, TreeBuilder treeBuilder,
    public FunctionParameter(Node parent, Tree t, TreeBuilder treeBuilder,
    // END EXTENSION
            SelectStatement selectStatement) throws TreeParsingException {
		// EXTENSION
		// ISSUE: DVCORE-2780
		// FIXED BY: Roman Pedchenko
		// DATE: 2015.04.09
		// this.selectStatement = selectStatement;
        this.parentNode = parent;
		this.selectStatement = selectStatement;
		// END EXTENSION
        this.builder = treeBuilder;
        // EXTENSION
        // ISSUE: DVCORE-2780
        // FIXED BY: Roman Pedchenko
        // DATE: 2015.04.09
        if (parentNode instanceof Expression)
        	this.uniqueId = this.builder.getuniqueid();
        // END EXTENSION
        this.build(t, this.builder);
    }

    /**
     * Builder to parse out the ANTLR tree
     *
     * @param t - the ANTLR tree
     * @param builder - TreeBuilder for the helper functions
     * @throws TreeParsingException
     */
    public void build(Tree t, TreeBuilder builder) throws TreeParsingException {
        // EXTENSION
        // ISSUE: DVCORE-2780
        // FIXED BY: Roman Pedchenko
        // DATE: 2015.04.09
        // if (t.getType() == JdbcGrammarParser.FUNCTIONPARAMETERS) {
        if (t.getType() == JdbcGrammarParser.SINGLEEXPRESSION) {
        // END EXTENSION
            this.tokenName = JdbcGrammarParser.tokenNames[t.getType()];
            this.tokenType = t.getType();
            this.logger.debug("BUILDING " + this.tokenName);
            for (int i = 0; i < t.getChildCount(); i++) {
                Tree child = t.getChild(i);
                switch (child.getType()) {
                    case JdbcGrammarParser.INTEGERPARAM:
                        this.refObject = new StringLiteral(child.getChild(0)
                                // EXTENSION
                                // ISSUE: DVCORE-2293
                                // FIXED BY: Roman Pedchenko
                                // DATE: 2014.10.13
//                                .getText());
                        		.getText(), JdbcGrammarParser.INTEGERPARAM);
                        		// END EXTENSION
                    	// EXTENSION
                    	// ISSUE: DVCORE-2780
                    	// FIXED BY: Roman Pedchenko
                    	// DATE: 2015.04.09
                    	this.children.addLast(refObject);
                    	// END EXTENSION
                        break;
// EXTENSION
// ISSUE: DVCORE-2780
// FIXED BY: Roman Pedchenko
// DATE: 2015.04.09
//                     // EXTENSION
//                     // ISSUE: DVCORE-2293
//                     // FIXED BY: Roman Pedchenko
//                     // DATE: 2014.10.13
//                     case JdbcGrammarParser.FLOATPARAM:
//                         this.refObject = new StringLiteral(child.getChild(0)
//                         		.getText(), JdbcGrammarParser.FLOATPARAM);
//                         break;
//                		// END EXTENSION
// END EXTENSION
                    case JdbcGrammarParser.STRINGLIT:
                        this.refObject = new StringLiteral(BQSupportFuncts.unquoteLiteral(child.getChild(0)
                                // EXTENSION
                                // ISSUE: DVCORE-2293,DVCORE-2473
                                // FIXED BY: Roman Pedchenko
                                // DATE: 2014.10.13,2014.10.31
//                                .getText());
                        		.getText()), JdbcGrammarParser.STRINGLIT);
                        		// END EXTENSION
                    	// EXTENSION
                    	// ISSUE: DVCORE-2780
                    	// FIXED BY: Roman Pedchenko
                    	// DATE: 2015.04.09
                    	this.children.addLast(refObject);
                    	// END EXTENSION
                        break;
                    case JdbcGrammarParser.COLUMN:
                        this.refObject = new ColumnReference(child, builder,
                                this.selectStatement,this);
                    	// EXTENSION
                    	// ISSUE: DVCORE-2780
                    	// FIXED BY: Roman Pedchenko
                    	// DATE: 2015.04.09
                    	this.children.addLast(refObject);
                    	// END EXTENSION
                        break;
                    case JdbcGrammarParser.DTTYPE:
                        this.refObject = new StringLiteral(child.getChild(0).getText(), true);
                    	this.children.addLast(refObject);
                        break;
                    case JdbcGrammarParser.JOKER:
                        // EXTENSION
                        // ISSUE: DVCORE-2293
                        // FIXED BY: Roman Pedchenko
                        // DATE: 2014.10.13
//                        this.refObject = new StringLiteral("*");
                        // EXTENSION
                        // ISSUE: DVCORE-2780
                        // FIXED BY: Roman Pedchenko
                        // DATE: 2015.04.09
                        // this.refObject = new StringLiteral("*", JdbcGrammarParser.STRINGLIT);
                        this.refObject = new StringLiteral("*", true);
                        // END EXTENSION
                        // END EXTENSION
                    	// EXTENSION
                    	// ISSUE: DVCORE-2780
                    	// FIXED BY: Roman Pedchenko
                    	// DATE: 2015.04.09
                    	this.children.addLast(refObject);
                    	// END EXTENSION
                        break;
                    // EXTENSION
                    // ISSUE: DVCORE-2780
                    // FIXED BY: Roman Pedchenko
                    // DATE: 2015.04.09
                    case JdbcGrammarParser.WITHINRECORD:
                        this.withinrecord = true;
                        break;
                    case JdbcGrammarParser.ALIAS:
                        this.alias = child.getChild(0).getText();
                        break;
                    case JdbcGrammarParser.FUNCTIONCALL:
                    	this.refObject = new FunctionCall(child,
                                this.builder,
                                this.selectStatement, this);
                    	this.children.addLast(refObject);
                        break;
                    case JdbcGrammarParser.FLOATPARAM:
                        this.refObject = new StringLiteral(child.getChild(0)
                        		.getText(), JdbcGrammarParser.FLOATPARAM);
                    	this.children.addLast(refObject);
                        break;
                    case JdbcGrammarParser.LPARAM:
                        this.refObject = new StringLiteral("(", true);
                    	this.children.addLast(refObject);
                        break;
                    case JdbcGrammarParser.RPARAM:
                        this.refObject = new StringLiteral(")", true);
                    	this.children.addLast(refObject);
                        break;
                    case JdbcGrammarParser.COMPARISONOPERATOR:
                    case JdbcGrammarParser.EXPRESSIONOPERATOR:
                    case JdbcGrammarParser.NULLNODE:
                    case JdbcGrammarParser.BOOLEANPARAM:
                        this.refObject = new StringLiteral(child.getChild(0).getText(), true);
                    	this.children.addLast(refObject);
                        break;
                    // END EXTENSION
                    default:
                        break;
                }
            }
            // EXTENSION
            // ISSUE: DVCORE-2780
            // FIXED BY: Roman Pedchenko
            // DATE: 2015.04.09
            if (this.getAlias() == null) {
                // we give alias as unique id
                this.alias = this.uniqueId;
            }
            // END EXTENSION
        } else {
            throw new TreeParsingException(
                    "This Tree is not a FUNCTIONPARAMETER");
        }
    }

	// EXTENSION
	// ISSUE: DVCORE-2780
	// FIXED BY: Roman Pedchenko
	// DATE: 2015.04.09
	public ColumnReference getColumnReference() {
		if (children.size() == 1 && children.get(0) instanceof ColumnReference)
			return (ColumnReference) children.get(0);
		return null;
	}


	public FunctionCall getFunctionCall() {
		if (children.size() == 1 && children.get(0) instanceof FunctionCall)
			return (FunctionCall) children.get(0);
		return null;
	}

	public Node simplify() {
		if (withinrecord)
			return null;
		if (children.size() == 1 && children.get(0) instanceof ColumnReference) {
			ColumnReference ref = (ColumnReference) children.get(0);
			ColumnCall col = new ColumnCall(ref.getPrefixes(), ref.getAliases() != null ? Arrays.asList(ref.getAliases()) : null, builder, ref.getName(), ref.getPointedNode(), (Expression) parentNode);
			return returnWithAlias(col);
		}
		if (children.size() == 1 && children.get(0) instanceof FunctionCall)
			return returnWithAlias((FunctionCall) children.get(0));
		return null;
	}

	private String getActualAlias() {
		if (!this.alias.equals(this.uniqueId))
			return this.alias;
		return null;
	}

	private Node returnWithAlias(ColumnCall node) {
		String aliasToSet = getActualAlias();
		if (aliasToSet != null)
			node.setAlias(aliasToSet);
		else
			node.setAlias(getUniqueid());
		node.setUniqueId(getUniqueid());
		return node;
	}

	private Node returnWithAlias(FunctionCall node) {
		node.setParentNode(parentNode);
		String aliasToSet = getActualAlias();
		if (aliasToSet != null)
			node.setAlias(aliasToSet);
		else
			node.setAlias(getUniqueid());
		node.setUniqueId(getUniqueid());
		return node;
	}
	// END EXTENSION

	@Override
    public Node getMainNode() {
        return this;
    }

    // EXTENSION
    // ISSUE: DVCORE-2780
    // FIXED BY: Roman Pedchenko
    // DATE: 2015.04.09
    public String getAlias() {
        return this.alias;
    }
    // END EXTENSION

    /**
     * Getter for the function parameter which can be:
     * <li> INTEGERPARAM
     * <li> STRINGLIT
     * <li> COLUMN
     * <li> JOKER
     *
     * @return - A Node which type describe the parameters type
     */
    public Node getRefobject() {
        return this.refObject;
    }

    @Override
    public String toPrettyString() {
        return this.toPrettyString(-1);
    }

    @Override
    public String toPrettyString(int level) {
        // EXTENSION
        // ISSUE: DVCORE-2780
        // FIXED BY: Roman Pedchenko
        // DATE: 2015.04.09
        // return this.refObject.toPrettyString(level);
    	StringBuilder sb = new StringBuilder();
    	for (Node obj : children)
    		sb.append(' ').append(obj.toPrettyString(level));
        if (parentNode instanceof Expression) {
        	if (withinrecord)
        		sb.append(" WITHIN RECORD ");
        	sb.append(" AS " + this.uniqueId);
        }
        return sb.toString();
    }

	@Override
	public String getUniqueid() {
		return this.uniqueId;
	}

	public boolean isConst() {
		return this.children.size() == 1 && this.children.get(0) instanceof StringLiteral;
	}
	// END EXTENSION

	/**
	 * returns the synonyms a.k.a Alias of this function
	 * if there's a parent with an alias we also return
	 * the alias with its parents alias
	 *
	 * @return A Stringlist that contains the aliases, or null
	 */
	public List<String> getSynonyms()
	{
		List<String> returnList = new ArrayList<String>();
		SubQuery parent = this.selectStatement.parent;

		if(this.getAlias()!=null){
			returnList.add(this.getAlias());
			if(parent!=null && parent.getAlias()!=null){
				returnList.add(parent.getAlias()+"."+this.getAlias());
			}
			return returnList;
		}
		else
		{
			returnList.add(this.uniqueId);
			returnList.add(parent.getAlias()+"."+this.uniqueId);
			return returnList;
		}
	}
}
